package jp.ac.takushoku_u.cs;

/**
* ArrangePointクラスはグラフの要素を配置する情報(描画座標位置情報)を返すクラスです。<br>
* 非循環で双方向の枝のないのグラフ(隣接マトリクス)から情報を収得します。
* 使用するコンストラクタによっては各項目を配置するレベル情報とレベル内での位置情報
* (項目間の上下および左右関係の情報)が必要となります。
* @version 1.0
* @author 笠島 裕史(Kasajima Hiroshi)
*/
/*
* 最終更新日 2005年03月01日
*/
public class ArrangePoint{
	
/**
* 隣接マトリクス
*/
	private BooleanMatrix matrix;
	
/**
* 可到達マトリクス
*/
	private BooleanMatrix reachMatrix;
	
/**
* 最大レベル(レベルは0から始まるため実際のレベル数は最大レベル+1)
*/
	private int maxLevel;
	
/**
* 最大横幅数
*/
	private int maxWide;
	
/**
* 描画要素数
*/
	private int elementNumber;
	
/**
* 描画レベル配列(配列数は要素数)
*/
	private int[] paintLevel;
	
/**
* 同一レベルでの描画位置
*/
	private int[] paintNumber;
	
/**
* 横幅配列(配列数はレベル数)
*/
	private int[] paintWide;
	
/**
* 各要素のx,y座標
*/
	private int[][] paintPoint;
	
/**
* コンストラクタを抜けたかどうか
*/
	private boolean isCon = false;
	
/**
* 描画するときの1ブロックの大きさ
*/
	private int blockSize = 50;
	
	
/**
* 要素のない隣接マトリクスを設定します。
*/
	public ArrangePoint(){
		BooleanMatrix mat = new BooleanMatrix();
		setPaintArray();
		point();
		isCon = true;
	}
	
/**
* 隣接マトリクスを設定し、描画位置情報を求めます。
* @param adjacencyMatrix 隣接マトリクス
* @exception NotSquareMatrixException 設定したマトリクスが正方行列ではない場合
*/
	public ArrangePoint(BooleanMatrix adjacencyMatrix){
		setMatrix(adjacencyMatrix);
		setPaintArray();
		point();
		isCon = true;
	}
	
/**
* 隣接マトリクスとレベル情報を設定し、描画位置情報を求めます。
* @param adjacencyMatrix 隣接マトリクス
* @param levelArray 各要素をどのレベルに配置するかレベル番号を格納した配列
* @exception NotSquareMatrixException 設定したマトリクスが正方行列ではない場合
* @exception DisagreementArraySizeException マトリクスと配列のサイズが一致しない場合
* @see jp.ac.takushoku_u.cs.LFGPoint
* @see jp.ac.takushoku_u.cs.LFGPoint#getPaintLevel()
*/
	public ArrangePoint(BooleanMatrix adjacencyMatrix, int[] levelArray){
		setMatrix(adjacencyMatrix);
		setPaintArray(levelArray);
		point();
		isCon = true;
	}
	
/**
* 隣接マトリクスとレベル情報、同一レベルの要素間での配置情報を設定し、描画位置情報を求めます。
* クラスjp.ac.takushoku_u.cs.IMMethod を利用した場合はこのコンストラクタを用います。
* @param adjacencyMatrix 隣接マトリクス
* @param levelArray 各要素をどのレベルに配置するかレベル番号を格納した配列
* @param wideArray 各要素を同一レベルのどこに配置するかを格納した配列
* @exception NotSquareMatrixException 設定したマトリクスが正方行列ではない場合
* @exception DisagreementArraySizeException マトリクスと配列のサイズが一致しない場合
* @see jp.ac.takushoku_u.cs.IMMethod
* @see jp.ac.takushoku_u.cs.IMMethod#getPaintLevel()
* @see jp.ac.takushoku_u.cs.IMMethod#getPaintWideNumber()
* @see jp.ac.takushoku_u.cs.LFGPoint
* @see jp.ac.takushoku_u.cs.LFGPoint#getPaintLevel()
	* @see jp.ac.takushoku_u.cs.LFGPoint#getPaintWideNumber()
*/
	public ArrangePoint(BooleanMatrix adjacencyMatrix, int[] levelArray, int[] wideArray){
		setMatrix(adjacencyMatrix);
		setPaintArray(levelArray, wideArray);
		point();
		isCon = true;
	}
	
	
/**
* 描画座標位置の配列を決めます。
*/
	private void point(){
		int center = (maxWide+1) * blockSize / 2;
		
		//各レベルで既に描いた要素数をカウントします。
		int[] countArray = new int [maxLevel+1];
		
		//センタリング
		for(int i = 0; i <= maxLevel; i++){
			if(maxWide%2==0){
				countArray[i] = (maxWide + 1 - paintWide[i]) / 2;
			}
			else{
				countArray[i] = (maxWide - paintWide[i]) / 2;
			}
		}
		
		
		//各要素のx座標とy座標を決めます
		for(int i = 0; i < elementNumber; i++){
			//横が偶数個
			if(paintWide[paintLevel[i]]%2 == 0){
				if(paintNumber[i] > paintWide[paintLevel[i]] / 2){
					paintPoint[i][0] = (countArray[paintLevel[i]] + paintNumber[i] + 1) * blockSize;
				}
				else{
					paintPoint[i][0] = (countArray[paintLevel[i]] + paintNumber[i]) * blockSize;
				}
			}
			
			//横が奇数個
			else{
				paintPoint[i][0] = (countArray[paintLevel[i]] + paintNumber[i]) * blockSize;
			}
			
			//y座標の高さ
			paintPoint[i][1] = (paintLevel[i]+1) * blockSize;
		}
	}
	
/**
* どこに配置するか座標(x,y)を指定した配列を返します。
* この配列が描画座標位置情報となります。
* @return 座標を指定した配列
*/
	public int[][] getPointArray(){
		point();
		return paintPoint;
	}
	
/**
* 描画を行う際の最大レベルを返します。
* @return 最大レベル
*/
	public int getMaxLevel(){
		return maxLevel;
	}
	
/**
* 描画を行う際の項目の最大横幅数を返します。
* 最大横幅は同一レベルにおいて項目数が一番多い数となります。
* @return 最大横幅数
*/
	public int getMaxWide(){
		return maxWide;
	}
	
/**
* 描画要素数(項目数)を返します。
* @return 要素数
*/
	public int getElementNumber(){
		return elementNumber;
	}
	
/**
* グラフを描く際の各要素のレベルを配列として返します。
* @return レベルを格納した配列
*/
	public int[] getPaintLevel(){
		return paintLevel;
	}
	
/**
* グラフを描く際の同一レベルでの位置番号を配列として返します。
* 配列内は同一レベル内で番号が小さい項目の方をより左に描くとします。
* @return 位置を格納した配列
*/
	public int[] getPaintWideNumber(){
		return paintNumber;
	}
	
/**
* 各レベルの横幅情報を保持した配列を返します。
* 各要素はその項目番号のレベルに横にいくつ項目を描画するかを示します。
* @return 横幅情報の配列
*/
	public int[] getPaintWide(){
		return paintWide;
	}
	
/**
* 最大レベルを探索します。
* @param levelArray レベルが入った配列
* @return 最大レベル
*/
	private int getRowMaxLevel(int[] levelArray){
		int num = 0;
		for(int i = 0; i < levelArray.length; i++){
			if(levelArray[i] > num){
				num = levelArray[i];
			}
		}
		return num;
	}
	
/**
* 最大横幅を探索します。
* @param levelArray レベルが入った配列
* @return 最大横幅
*/
	private int getColMaxLevel(int[] levelArray){
		int maxCount = 0;
		for(int i = 0; i < levelArray.length; i++){
			int count = 0;
			for(int j = 0; j < levelArray.length; j++){
				if(levelArray[j] == i){
					count++;
				}
			}
			if(count > maxCount){
				maxCount = count;
			}
		}
		
		return maxCount;
	}
	
/**
* 指定したレベルの横幅を探索します。
* @param level 指定するレベル
* @param levelArray レベルが入った配列
* @parama NumberArray レベル内でどこに配置するかを格納した配列
* @return 横幅
*/
	private int getColLevel(int level, int[] levelArray, int[] numberArray){
		int count = 0;
		for(int i = 0; i < levelArray.length; i++){
			if(levelArray[i] == level && count < numberArray[i]){
				count = numberArray[i];
			}
		}
		
		return count;
	}
	
/**
* 隣接マトリクスを設定します。
* @param adjacencyMatrix 隣接マトリクス
* @exception NotSquareMatrixException 設定したマトリクスが正方行列ではない場合
*/
	public void setMatrix(BooleanMatrix adjacencyMatrix){
		matrix = new BooleanMatrix();
		reachMatrix = new BooleanMatrix();
		if(adjacencyMatrix.isSquare()){
			matrix.matrixCopy(adjacencyMatrix);
			reachMatrix.matrixCopy(matrix.getReachMatrix());
			if(isCon){
				setPaintArray();
			}
		}
		else{
			throw new NotSquareMatrixException("row("+ adjacencyMatrix.getRowLength() +
				"),col("+adjacencyMatrix.getColLength()+")");
		}
	}
	
/**
* 描画情報のための配列を設定します。
*/
	private void setPaintArray(){
		int[] returnArray = new int[matrix.getRowLength()];
		System.arraycopy(getLevelArray(), 0, returnArray, 0, matrix.getRowLength());
		
		setPaintArray(returnArray);
	}
	
/**
* グラフを描く際のレベルを配列として返します。
* @return レベルを格納した配列
*/
	private int[] getLevelArray(){
		int[] returnArray = new int[matrix.getRowLength()];
		BooleanMatrix reachMatrix= new BooleanMatrix();
		reachMatrix.matrixCopy(matrix.getReachMatrix());
		
		//配列の初期化
		for(int i = 0; i < returnArray.length; i++){
			returnArray[i] = -1;
		}
		
		//ルートを決める
		for(int i = 0; i < returnArray.length; i++){
			if(matrix.getRowSum(i) == 0){
				returnArray[i] = 0;
			}
		}
		
		boolean flag = true;
		
		//レベルを格納する
		for(int level = 0; level < returnArray.length && flag; level++){
			for(int i = 0; i < returnArray.length; i++){
				
				//レベルが決定していなければ
				if(returnArray[i] == -1){
					int count = 1;
					for(int j = 0; j < returnArray.length; j++){
						
						//枝の先の要素はレベルが決定しているか
						if(reachMatrix.getCell(i, j) && returnArray[j] >= 0 &&
							returnArray[j] < level){
							count++;
						}
						else if(matrix.getCell(i, j) && matrix.getCell(j, i)){
							count++;
						}
					}
					if(count == reachMatrix.getRowSum(i)){
						returnArray[i] = level;
					}
				}
			}
		}
		
		//ルートの位置を変更する
		for(int i = 0; i < returnArray.length; i++){
			if(returnArray[i] == 0){
				int count = 0;
				for(int j = 0; j < matrix.getRowLength(); j++){
					if(matrix.getCell(j,i)){
						if(count < returnArray[j]){
							count = returnArray[j];
						}
					}
				}
				returnArray[i] = count - 1;
			}
		}
		
		return returnArray;
	}
	
/**
* 描画情報のための配列を設定します。
* @param levelArray 各要素をどのレベルに配置するかレベル番号を格納した配列
*/
	private void setPaintArray(int[] levelArray){
		int[] returnArray = new int[matrix.getRowLength()];
		System.arraycopy(getWideNumber(levelArray), 0, returnArray, 0, matrix.getRowLength());
		setPaintArray(levelArray, returnArray);
	}
	
/**
* 各レベルでの配置順を配列として返します。
* @param levelArray レベルを格納した配列
* @return 配置順を格納した配列
*/
	private int[] getWideNumber(int[] levelArray){
		int maxWide = getRowMaxLevel(levelArray);
		int[] wide = new int[maxWide+1];
		for(int i = 0; i < wide.length; i++){
			wide[i] = 1;
		}
		
		int[] returnArray = new int[matrix.getRowLength()];
		for(int i = 0; i < matrix.getRowLength(); i++){
			returnArray[i] = wide[levelArray[i]];
			wide[levelArray[i]]++;
		}
		return returnArray;
	}
	
/**
* 描画情報のための配列を設定します。
* @param levelArray 各要素をどのレベルに配置するかレベル番号を格納した配列
* @param wideArray 各要素を同一レベル内のどこに配置するか位置番号を格納した配列
* @exception DisagreementArraySizeException マトリクスと配列のサイズが一致しない場合
*/
	public void setPaintArray(int[] levelArray, int[] wideArray){
		if(matrix.getRowLength() == levelArray.length){
			maxLevel = getRowMaxLevel(levelArray);
			maxWide = getColMaxLevel(levelArray);
			elementNumber = levelArray.length;
			
			paintLevel = new int[levelArray.length];
			System.arraycopy(levelArray, 0, paintLevel, 0, levelArray.length);
			
			paintNumber = new int[wideArray.length];
			System.arraycopy(wideArray, 0, paintNumber, 0, wideArray.length);
			
			paintWide = new int[maxLevel+1];
			for(int i = 0; i <= maxLevel; i++){
				paintWide[i] = getColLevel(i, levelArray, wideArray);
			}
			
			paintPoint = new int[elementNumber][2];
			
		}
		else{
			throw new DisagreementArraySizeException("matrix("+matrix.getRowLength()+
				"),array("+levelArray.length+")");
		}
	}
	
}