package jp.ac.takushoku_u.cs;

/**
* LFGpointクラスはLFGを描画する際の配置レベルを決定するクラスです。
* 同一レベルでの配置順は、要素番号の小さいものから順に左から配置されます。
* このため、このクラスを用いて描画を行った場合有向枝の交差が起こる場合があります。
* @version 1.0
* @author 笠島 裕史(Kasajima Hiroshi)
*/
/*
* 最終更新日 2005年03月01日
*/
public class LFGPoint{
	
/**
* 隣接マトリクス
*/
	private BooleanMatrix matrix;
	
/**
* ルートとする項目
*/
	private int rootItem = -1;

/**
* 要素のない隣接マトリクスを設定します。
*/
	public LFGPoint(){
		matrix = new BooleanMatrix();
	}
	
/**
* 隣接マトリクスからグラフを描くためのレベル(縦、横軸情報)を求めます。
* @param adjacencyMatrix 隣接マトリクス
* @exception NotSquareMatrixException 設定したマトリクスが正方でない場合
*/
	public LFGPoint(BooleanMatrix adjacencyMatrix){
		matrix = new BooleanMatrix();
		if(adjacencyMatrix.isSquare()){
			matrix.matrixCopy(adjacencyMatrix);
		}
		else{
			throw new NotSquareMatrixException();
		}
	}
	
/**
* 隣接マトリクスからグラフを描くためのレベル(縦、横軸情報)を求めます。
* @param adjacencyMatrix 隣接マトリクス
* @param top ルートの要素
* @exception NotSquareMatrixException 設定したマトリクスが正方でない場合
* @exception NegativeValueException ルートの要素に負を指定した場合
*/
	public LFGPoint(BooleanMatrix adjacencyMatrix, int top){
		matrix = new BooleanMatrix();
		if(adjacencyMatrix.isSquare()){
			if(top < 0){
				matrix.matrixCopy(adjacencyMatrix);
				top = rootItem;
			}
			else{
				throw new NegativeValueException("top("+top+")");
			}
		}
		else{
			throw new NotSquareMatrixException();
		}
	}
	
/**
* 隣接マトリクスと縦軸情報からグラフを描くためのレベルのグループ(横軸情報)を求めます。
* @param adjacencyMatrix 隣接マトリクス
* @param averageArray 縦軸情報
* @exception NotSquareMatrixException 設定したマトリクスが正方でない場合
*/
	public LFGPoint(BooleanMatrix adjacencyMatrix, double[] averageArray){
		matrix = new BooleanMatrix();
		if(adjacencyMatrix.isSquare()){
			matrix.matrixCopy(adjacencyMatrix);
			
			double max = Double.NEGATIVE_INFINITY;
			int item = -1;
			for(int i = 0; i < averageArray.length; i++){
				if(averageArray[i] > max){
					max = averageArray[i];
					item = i;
				}
			}
			rootItem = item;
			
		}
		else{
			throw new NotSquareMatrixException();
		}
	}
	
	
/**
* グラフを描く際のレベルを配列として返します。
* @return レベルを格納した配列
*/
	public int[] getPaintLevel(){
		int[] returnArray = new int[matrix.getRowLength()];
		BooleanMatrix reachMatrix= new BooleanMatrix();
		reachMatrix.matrixCopy(matrix.getReachMatrix());
		
		//配列の初期化
		for(int i = 0; i < returnArray.length; i++){
			returnArray[i] = -1;
		}
		
		//ルートを決める
		if(rootItem == -1){
			for(int i = 0; i < returnArray.length; i++){
				if(matrix.getRowSum(i) == 0){
					returnArray[i] = 0;
				}
			}
		}
		else{
			returnArray[rootItem] = 0;
		}
		
		boolean flag = true;
		
		//レベルを格納する
		for(int level = 0; level < returnArray.length && flag; level++){
			for(int i = 0; i < returnArray.length; i++){
				
				//レベルが決定していなければ
				if(returnArray[i] == -1){
					int count = 1;
					for(int j = 0; j < returnArray.length; j++){
						
						//枝の先の要素はレベルが決定しているか
						if(reachMatrix.getCell(i, j) && returnArray[j] >= 0 &&
							returnArray[j] < level){
							count++;
						}
						else if(matrix.getCell(i, j) && matrix.getCell(j, i)){
							count++;
						}
					}
					if(count == reachMatrix.getRowSum(i)){
						returnArray[i] = level;
					}
				}
			}
		}
		
		//ルートの位置を変更する
		for(int i = 0; i < returnArray.length; i++){
			if(returnArray[i] == 0){
				int count = 0;
				for(int j = 0; j < matrix.getRowLength(); j++){
					if(matrix.getCell(j,i)){
						if(count < returnArray[j]){
							count = returnArray[j];
						}
					}
				}
				returnArray[i] = count - 1;
			}
		}
		
		return returnArray;
	}
	
/**
* 各レベルでの配置順を配列として返します。
* @return 配置順を格納した配列
*/
	public int[] getPaintWideNumber(){
		int[] levelArray = new int[matrix.getRowLength()];
		System.arraycopy(getPaintLevel(), 0, levelArray, 0, matrix.getRowLength());
		int maxWide = getRowMaxLevel(levelArray);
		int[] wide = new int[maxWide+1];
		for(int i = 0; i < wide.length; i++){
			wide[i] = 1;
		}
		
		int[] returnArray = new int[matrix.getRowLength()];
		for(int i = 0; i < matrix.getRowLength(); i++){
			returnArray[i] = wide[levelArray[i]];
			wide[levelArray[i]]++;
		}
		return returnArray;
	}
	
/**
* 各レベルでの配置順を配列として返します。
* @param levelArray レベルを格納した配列
* @return 配置順を格納した配列
*/
	public int[] getPaintWideNumber(int[] levelArray){
		int maxWide = getRowMaxLevel(levelArray);
		int[] wide = new int[maxWide+1];
		for(int i = 0; i < wide.length; i++){
			wide[i] = 1;
		}
		
		int[] returnArray = new int[matrix.getRowLength()];
		for(int i = 0; i < matrix.getRowLength(); i++){
			returnArray[i] = wide[levelArray[i]];
			wide[levelArray[i]]++;
		}
		return returnArray;
	}
	
/**
* 最大レベルを探索します。
* @param levelArray レベルが入った配列
* @return 最大レベル
*/
	private int getRowMaxLevel(int[] levelArray){
		int num = 0;
		for(int i = 0; i < levelArray.length; i++){
			if(levelArray[i] > num){
				num = levelArray[i];
			}
		}
		return num;
	}
	
/**
* 設定された縦軸情報からより上に描く順序を求めます。
* @param averageArray 縦軸情報
* @return 要素を上から描く順に項目番号を並び替えた配列。
*/
	private int[] getGraphArray(double[] averageArray){
		int[] returnArray = new int[matrix.getColLength()];		//平均評定値の高い順に並べた時の列番号（項目）を入れる
		
		for (int i = 0; i < averageArray.length; i++){
			double maxAverage = averageArray[0];		//初期値
			int maxNumber = 0;
			for (int j = 1; j <  averageArray.length; j++){		//比較を行う
				if (maxAverage < averageArray[j]){
					maxAverage = averageArray[j];
					maxNumber = j;
				}
			}
			averageArray[maxNumber] = Double.NEGATIVE_INFINITY;	 //比較対象からはずす
			returnArray[i] = maxNumber;
		}
		return returnArray;
	}
}