package jp.ac.takushoku_u.cs;
import java.util.Stack;

/**
* TaskSequenceMatrixクラスは系列結果を全て求めるためのクラスです。
* 系列結果はマトリクスで返され、各行がそれぞれ1つの系列結果となっています。
* そのため、返されるマトリクスの大きさは、系列結果×系列を行う項目数 となります。
* @version 1.0
* @author 笠島 裕史(Kasajima Hiroshi)
* @see jp.ac.takushoku_u.cs.TaskSequence
*/
/*
* 最終更新日 2005年02月08日
*/
public final class TaskSequenceMatrix{
	
/**
* コンストラクタ
*/
	private TaskSequenceMatrix(){
	}
	
/**
* 系列結果を全て求めたマトリクスを返します。
* マトリクスの中身は隣接マトリクスの列番号(系列化を行う項目番号)が、
* マトリクスの列番号の小さいほうから系列を行った順に入ります。
* @param adjacencyMatrix 系列化を行う隣接マトリクス
* @param alpha 指定するアルファの値
* @param beta 指定するベータの値
* @return 系列結果を保持したマトリクス
* @exception NotSquareMatrixException 隣接マトリクスが正方行列ではない場合
* @exception SetValueOutOfBoundsException アルファとベータの値が不正である場合
*/
	public static final ArithmeticMatrix getTaskMatrix(ArithmeticMatrix adjacencyMatrix, double alpha, double beta){
		TaskSequence ts = new TaskSequence(adjacencyMatrix);
		ArithmeticMatrix mat = new ArithmeticMatrix();
		mat.matrixCopy(getTaskMatrix(adjacencyMatrix, alpha, beta, ts.getTaskSequence(alpha, beta),
			ts.isDummy() ,ts.getDivergenceMatrix()));
		
		//0しかない行を排除する
		boolean isIn[] = new boolean[mat.getRowLength()];
		for(int i = 0; i < mat.getRowLength(); i++){
			int count = 0;
			for(int j = 0; j < mat.getColLength(); j++){
				if(mat.getCell(i, j) == 0){
					count++;
				}
			}
			if(count > 1){
				isIn[i] = false;
			}
			
			//0だけでない行
			else{
				isIn[i] = true;
			}
		}
		
		//0だけの行を埋める
		int number = -1;
		int counter = 0;
		for(int i = 0; i < isIn.length; i++){
			if(isIn[i]){
				number++;
				counter++;
				if(number != i){
					for(int j = 0; j < mat.getColLength(); j++){
						mat.setCell(number, j, mat.getCell(i,j));
						mat.setCell(i, j, 0);
					}
					isIn[i] = false;
					i = number + 1;
				}
			}
		}
		
		//リサイズ
		mat.reSizeMatrix(counter, mat.getColLength());
		mat.setRowNameNumber();
		mat.setColNameNumber();
		
		return mat;
	}
	
	
/**
* 系列結果を全て求めたマトリクスをStringMatrixで返します。
* マトリクスの中身は隣接マトリクスで指定した列名(系列化を行う項目名)が、
* マトリクスの列番号の小さい方から系列を行った順に入ります。
* @param adjacencyMatrix 系列化を行う隣接マトリクス
* @param alpha 指定するアルファの値
* @param beta 指定するベータの値
* @return 系列結果を保持したマトリクス
* @exception NotSquareMatrixException 隣接マトリクスが正方行列ではない場合
* @exception SetValueOutOfBoundsException アルファとベータの値が不正である場合
*/
	public static final StringMatrix getTaskStringMatrix(ArithmeticMatrix adjacencyMatrix, double alpha, double beta){
		ArithmeticMatrix mat = new ArithmeticMatrix();
		mat.matrixCopy(getTaskMatrix(adjacencyMatrix, alpha, beta));
		StringMatrix returnMatrix = new StringMatrix(mat.getRowLength(), mat.getColLength());
		
		for(int i = 0; i < mat.getRowLength(); i++){
			for(int j = 0; j < mat.getColLength(); j++){
				returnMatrix.setCell(i, j, adjacencyMatrix.getColName((int)mat.getCell(i, j)));
			}
		}
		
		return returnMatrix;
	}
	
/**
* 系列結果を全て求めます。
* @param matrix 系列化を行うマトリクス
* @param alpha 指定するアルファの値
* @param beta 指定するベータの値
* @param seq 系列化を行った系列
* @param du 系列を行う際にダミーを付けたかどうか
* @param trMatrix 分岐点を保持したマトリクス
* @return 系列結果のマトリクス
*/
		private static final ArithmeticMatrix getTaskMatrix(ArithmeticMatrix matrix, double alpha, double beta,
			Sequence seq, boolean du, BooleanMatrix trMatrix){
			
			ArithmeticMatrix returnMatrix = new ArithmeticMatrix(1, matrix.getColLength());
			Stack st = new Stack();
				
			//優先配列
			int[] ar = new int[0];
				
			//一次保存用配列
			int[] tmp = new int[matrix.getRowLength() * matrix.getRowLength()];
				
			//重複要素を排除した優先配列
			int[] tar = new int[0];
			TaskSequence tsq = new TaskSequence(matrix);
			
			//系列順をマトリクスに入れる
			for(int i = 0; i < seq.getLength(); i++){
				returnMatrix.setCell(0, i, seq.getItem(i));
			}
			
			for(int i = 0; i < trMatrix.getRowLength(); i++){
				for(int j = 0; j < trMatrix.getColLength(); j++){
					if(trMatrix.getCell(i, j)){
						if(du){
							st.push(new Edge(i-1, j-1));
						}
						else{
							st.push(new Edge(i, j));
						}
					}
				}
			}
			
			while(!st.empty()){
				
				//スタックから値を取り出す
				Edge ed = new Edge();
				ed = (Edge)st.pop();
				
				//優先配列を生成
				int element = ar.length;
				System.arraycopy(ar,0,tmp,0,element);
				ar = new int[element + 2];
				System.arraycopy(tmp,0,ar,0,element);
				
				//優先配列にスタックの値を入れる
				ar[element] = ed.getEnd();
				ar[element+1] = ed.getBegin();
				
				//重複要素を排除する
				int k = 0;
				for(int i=0; i < ar.length; i++){
					boolean flag = true;
					for(int j = i+1; j < ar.length; j++){
						if(ar[i] == ar[j]){
							flag = false;
						}
					}
					if(flag){
						tmp[k] = ar[i];
						k++;
					}
				}
				
				//系列化を行う際の優先配列を生成する
				tar = new int[k];
				System.arraycopy(tmp,0,tar,0,k);
				
				//系列化を行う(再帰)
				tsq.setPriorityOrder(tar);
				ArithmeticMatrix mat = new ArithmeticMatrix();
				mat.matrixCopy(getTaskMatrix(matrix, alpha, beta, 
					tsq.getTaskSequence(alpha, beta), tsq.isDummy(), tsq.getDivergenceMatrix(), ar));
				
				//系列結果を入れる
				int size = returnMatrix.getRowLength();
				int tsize = size;
				returnMatrix.reSizeMatrix(size + mat.getRowLength(), returnMatrix.getColLength());
				for(int i = 0; i < mat.getRowLength();i++){
					boolean flagB = true;
					for(int ia = 0; ia < size && flagB; ia++){
						boolean flagC = true;
						for(int ib = 0; ib < returnMatrix.getColLength() && flagC; ib++){
							if(returnMatrix.getCell(ia,ib) != mat.getCell(i,ib)){
								flagC = false;
							}
						}
						
						//flagCがtrueなら同じ系列がすでにある
						if(flagC){
							flagB = false;
						}
					}
					
					//同じ系列結果は排除
					if(flagB){
						for(int j = 0; j < mat.getColLength(); j++){
							returnMatrix.setCell(size + i, j, mat.getCell(i,j));
						}
						tsize++;
					}
				}
				
				returnMatrix.reSizeMatrix(tsize+1, returnMatrix.getColLength());
				
				//優先配列から後ろの2要素を削除
				element = ar.length;
				System.arraycopy(ar,0,tmp,0,element);
				ar = new int[element - 2];
				System.arraycopy(tmp,0,ar,0,element - 2);
				
			}
			return returnMatrix;
			
		}
	
/**
* 系列結果を全て求めます。
* @param matrix 系列化を行うマトリクス
* @param alpha 指定するアルファの値
* @param beta 指定するベータの値
* @param seq 系列化を行った系列
* @param du 系列を行う際にダミーを付けたかどうか
* @param trMatrix 分岐点を保持したマトリクス
* @param ar 優先順を保持した配列
* @return 系列結果のマトリクス
*/
		private static final ArithmeticMatrix getTaskMatrix(ArithmeticMatrix matrix, double alpha, double beta,
			Sequence seq, boolean du, BooleanMatrix trMatrix, int[] ar){
			
			
			ArithmeticMatrix returnMatrix = new ArithmeticMatrix(1, matrix.getColLength());
			Stack st = new Stack();
				
			//一次保存用配列
			int[] tmp = new int[matrix.getRowLength() * matrix.getRowLength()];
			int[] tar = new int[0];
			TaskSequence tsq = new TaskSequence(matrix);
			
			//系列順をマトリクスに入れる
			for(int i = 0; i < seq.getLength(); i++){
				returnMatrix.setCell(0, i, seq.getItem(i));
			}
			
			for(int i = 0; i < trMatrix.getRowLength(); i++){
				for(int j = 0; j < trMatrix.getColLength(); j++){
					if(trMatrix.getCell(i, j)){
						boolean flagA = true;
						
						//優先配列にある要素は除外
						for(int n = 0; n < ar.length; n = n + 2){
							if(ar[n] == j && ar[n+1] == i && !du){
								flagA = false;
							}
							else if(ar[n] == j-1 && ar[n+1] == i-1 && du){
								flagA = false;
							}
						}
						if(flagA){
							if(du){
								st.push(new Edge(i-1, j-1));
							}
							else{
								st.push(new Edge(i, j));
							}
						}
					}
				}
			}
			
			while(!st.empty()){
				
				//スタックから値を取り出す
				Edge ed = new Edge();
				ed = (Edge)st.pop();
				
				//優先配列を生成
				int element = ar.length;
				System.arraycopy(ar,0,tmp,0,element);
				ar = new int[element + 2];
				System.arraycopy(tmp,0,ar,0,element);
				
				//優先配列にスタックの値を入れる
				ar[element] = ed.getEnd();
				ar[element+1] = ed.getBegin();
				
				//重複要素を排除する
				int k = 0;
				for(int i=0; i < ar.length; i++){
					boolean flag = true;
					for(int j = i+1; j < ar.length; j++){
						if(ar[i] == ar[j]){
							flag = false;
						}
					}
					if(flag){
						tmp[k] = ar[i];
						k++;
					}
				}
				
				//系列化を行う際の優先配列を生成する
				tar = new int[k];
				System.arraycopy(tmp,0,tar,0,k);
				
				//系列化を行う(再帰)
				tsq.setPriorityOrder(tar);
				ArithmeticMatrix mat = new ArithmeticMatrix();
				mat.matrixCopy(getTaskMatrix(matrix, alpha, beta, 
					tsq.getTaskSequence(alpha, beta), tsq.isDummy(), tsq.getDivergenceMatrix(), ar));
				
				
				//系列結果を入れる
				int size = returnMatrix.getRowLength();
				int tsize = size;
				
				returnMatrix.reSizeMatrix(size + mat.getRowLength(), returnMatrix.getColLength());
				
				for(int i = 0; i < mat.getRowLength();i++){
					
					boolean flagB = true;
					
					for(int ia = 0; ia < size && flagB; ia++){
						boolean flagC = true;
						for(int ib = 0; ib < returnMatrix.getColLength() && flagC; ib++){
							if(returnMatrix.getCell(ia,ib) != mat.getCell(i,ib)){
								flagC = false;
							}
						}
						
						//flagCがtrueなら同じ系列がすでにある
						if(flagC){
							flagB = false;
						}
					}
					
					//同じ系列結果は排除
					if(flagB){
						for(int j = 0; j < mat.getColLength(); j++){
							returnMatrix.setCell(size + i, j, mat.getCell(i,j));
						}
						tsize++;
					}
				}
				
				returnMatrix.reSizeMatrix(tsize+1, returnMatrix.getColLength());
				
				//優先配列から後ろの2要素を削除
				element = ar.length;
				System.arraycopy(ar,0,tmp,0,element);
				ar = new int[element - 2];
				System.arraycopy(tmp,0,ar,0,element - 2);
				
			}
			return returnMatrix;
			
		}
}